/*!
 *	\file main.c
 *	\author Didier Benoit <benoit@imnc.in2p3.fr>
 *	\date October 2011, QIM IMNC-IN2P3/CNRS, Paris VII-XI, Orsay
 *	\brief Convert 8 bits to unsigned 16 bits
 *	Usage: ./convert8bitsToU16bits <w> <h> <d> <in> <out>
 *	<w>    Width of your input file
 *	<h>    Height of your input file
 *	<d>    Depth of your input file
 *	<in>   Image to convert
 *	<out>  Converted image
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

void printHelpAndQuit( char const* msg )
{
	fprintf( stderr, "%s\n", msg );
	fprintf( stderr, "Usage: ./convert8bitsToU16bits <w> <h> <d> <in> <out>\n" );
	fprintf( stderr, "<w>    Width of your input file\n" );
	fprintf( stderr, "<h>    Height of your input file\n" );
	fprintf( stderr, "<d>    Depth of your input file\n" );
	fprintf( stderr, "<in>   Image to convert\n" );
	fprintf( stderr, "<out>  Converted image\n" );
	exit( EXIT_FAILURE );
}

int main( int argc, char** argv )
{
	// Checking the number of arguments
	if( argc < 6 )
	{
		char const* msg = "Argument(s) missing!!!";
		printHelpAndQuit( msg );
	}

	// Taking arguments
	unsigned int const WIDTH = atoi( argv[ 1 ] );
	unsigned int const HEIGHT = atoi( argv[ 2 ] );
	unsigned int const DEPTH = atoi( argv[ 3 ] );
	char const* INPUTFILENAME = argv[ 4 ];
	char const* OUTPUTFILENAME = argv[ 5 ];

	// Opening the input file, and read it in a buffer
	int fdInputOpen = open( INPUTFILENAME, O_RDONLY | O_NONBLOCK );
	if( fdInputOpen < 0 )
	{
		fprintf( stderr, "Error opening '%s': %s\n", INPUTFILENAME,
			strerror( errno ) );
		exit( EXIT_FAILURE );
	}

	// Allocating the buffer for the input
	char* inputBuffer = (char*)malloc( WIDTH * HEIGHT * DEPTH * sizeof( char ) );
	int fdInputRead = read( fdInputOpen, inputBuffer,
		WIDTH * HEIGHT * DEPTH * sizeof( char ) );
	if( fdInputRead < 0 )
  {
    fprintf( stderr, "Error opening '%s': %s\n", INPUTFILENAME,
      strerror( errno ) );
    exit( EXIT_FAILURE );
  }

	// Opening the output file, and read it in a buffer
  int fdOutputOpen = open( OUTPUTFILENAME, O_WRONLY | O_CREAT, 0644 );
  if( fdOutputOpen < 0 )
  {
    fprintf( stderr, "Error opening '%s': %s\n", OUTPUTFILENAME,
      strerror( errno ) );
    exit( EXIT_FAILURE );
  }

  // Allocating the buffer for the input
  unsigned short* outputBuffer = (unsigned short*)malloc(
		WIDTH * HEIGHT * DEPTH * sizeof( unsigned short ) );
  
	// Converting 8 bits to unsigned 16 bits
	for( unsigned int i = 0; i < WIDTH * HEIGHT * DEPTH; ++i )
	{
		*( outputBuffer + i ) = (unsigned short)( *( inputBuffer + i ) );
	}

	int fdOutputWrite = write( fdOutputOpen, outputBuffer,
    WIDTH * HEIGHT * DEPTH * sizeof( unsigned short ) );
  if( fdOutputWrite < 0 )
  {
    fprintf( stderr, "Error opening '%s': %s\n", OUTPUTFILENAME,
      strerror( errno ) );
    exit( EXIT_FAILURE );
  }

	// Freeing memory
	free( inputBuffer );
	free( outputBuffer );

	// Closing files
	int fdInputClose = close( fdInputOpen );
	if( fdInputClose < 0 )
	{
		fprintf( stderr, "Error closing '%s': %s\n", INPUTFILENAME,
			strerror( errno ) );
		exit( EXIT_FAILURE );
	}

	int fdOutputClose = close( fdOutputOpen );
  if( fdOutputClose < 0 )
  {
    fprintf( stderr, "Error closing '%s': %s\n", OUTPUTFILENAME,
      strerror( errno ) );
    exit( EXIT_FAILURE );
  }

	return 0;
}
